from fngradio.decorators import (  # noqa
    FnGradio,
    DEFAULT_FNGRADIO,
    interface
)
from fngradio.tabbed_interface import (  # noqa
    tabbed_interface
)
from fngradio.app import (  # noqa
    FnGradioApp
)



import pytest

import gradio as gr

from fngradio.app import FnGradioApp


@pytest.fixture(name='fngr_app')
def _fngr_app() -> FnGradioApp:
    return FnGradioApp()


class TestFnGradioApp:
    def test_should_create_gradio_interface_with_simple_type(
        self,
        fngr_app: FnGradioApp
    ):
        @fngr_app.interface()
        def fn(s: str) -> str:
            return s.upper()

        assert fn.input_components and len(fn.input_components) == 1
        assert isinstance(fn.input_components[0], gr.Textbox)
        assert len(fn.output_components) == 1
        assert isinstance(fn.output_components[0], gr.Textbox)

    def test_should_remember_interfaces(
        self,
        fngr_app: FnGradioApp
    ):
        @fngr_app.interface()
        def fn(s: str) -> str:
            return s.upper()

        interfaces = fngr_app.get_interfaces()
        assert interfaces == [fn]

    def test_should_create_tabbed_interface(
        self,
        fngr_app: FnGradioApp
    ):
        @fngr_app.interface()
        def fn(s: str) -> str:
            return s.upper()

        demo = fngr_app.tabbed()
        assert isinstance(demo, gr.TabbedInterface)

    def test_should_combine_apps(
        self
    ):
        app_1 = FnGradioApp()
        app_2 = FnGradioApp()

        @app_1.interface()
        def upper(s: str) -> str:
            return s.upper()

        @app_2.interface()
        def lower(s: str) -> str:
            return s.lower()

        fngr_app = FnGradioApp(apps=[app_1, app_2])
        assert fngr_app.get_interfaces() == (
            list(app_1.get_interfaces()) + list(app_2.get_interfaces())
        )



from typing import Sequence

import gradio as gr

from fngradio.decorators import FnGradio
from fngradio.tabbed_interface import tabbed_interface


class FnGradioApp(FnGradio):
    def __init__(self, apps: Sequence['FnGradioApp'] | None = None) -> None:
        super().__init__()
        self._interfaces: list[gr.Interface] = []
        if apps is not None:
            for app in apps:
                self._interfaces.extend(app._interfaces)

    def get_interfaces(self) -> Sequence[gr.Interface]:
        return self._interfaces

    def _on_interface(self, interface: gr.Interface):
        super()._on_interface(interface)
        self._interfaces.append(interface)

    def tabbed(
        self,
        tab_names: Sequence[str] | None = None,
        **kwargs
    ) -> gr.TabbedInterface:
        return tabbed_interface(
            self._interfaces,
            tab_names=tab_names,
            **kwargs
        )
